<?php
namespace Teknisa\EST\Controllers;

use Doctrine\ORM\EntityManager;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;

use Teknisa\EST\Controllers\TeknisaCrud;
use Teknisa\EST\Services\Ger01700UnidadeService;

class Ger01700UnidadeController extends TeknisaCrud {
    protected $dataSourceNames = array(
        'default'      => '',
        'filial'       => 'filial_view',
        'endefili'     => 'endefili_view',
        'almoxarifado' => 'almoxarifado',
        'localestoque' => 'localestoque',
        'paramfilial'  => 'paramfilial',
        'filipadr'     => 'paramfilipadr'
    );

    protected $entityManager;
    protected $environment;
    protected $service;
    protected $cripto;

    /**
     * Construtor do Controller.
     *
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param EntityManager $entityManager
     * @param Environment $environment
     * @param Ger01700UnidadeService $service
     */
    public function __construct($crudService, ParameterBag $parameterBag, EntityManager $entityManager, $environment, $cripto, Ger01700UnidadeService $service) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->service       = $service;
        $this->cripto        = $cripto;

        $this->switchDataSource('default');

        parent::__construct($crudService, $parameterBag);
    }

    /**
     * Procedimento que recebe um filtro e realiza a
     * requisição de atualização dos produtos para a
     * unidade e nível de produto informados. Capta a
     * organização do usuário logado.
     *
     * Envia no objeto response um DataSet com uma
     * mensagem de sucesso ou erro conforme necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function buildAtualizaProduto(DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();

        $CDFILIAL    = $filters[0]['value'];
        $CDNVPRODUTO = $filters[1]['value'];
        $NRORG       = $this->environment->getNrOrgTrab();

        try {
            $this->service->atualizaProduto($CDFILIAL, $CDNVPRODUTO, $NRORG);

            $response->addDataSet(new DataSet("success", array("MESSAGE" => "Produtos atualizados com sucesso para a unidade ".$CDFILIAL.".")));
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    /**
     * Procedimento que recebe um filtro e realiza a
     * requisição de validação de endereço para a unidade
     * informada. Capta a organização do usuário logado.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function buildValidationForEndefili(DTO\Request\Filter $request, DTO\Response $response) {
        $validation = null;

        $filters = $request->getFilterCriteria()->getConditions();

        $CDFILIAL = $filters[0]['value'];
        $NRORG    = $this->environment->getNrOrgTrab();

        try {
            $validation = $this->service->validateEndefili($CDFILIAL, $NRORG);

            $response->addDataSet(new DataSet("validation", $validation));
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    /**
     * Procedimento que realiza a requisição de validação
     * da tab de parametrização.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function buildValidationForParamfilipadr(DTO\Request\Filter $request, DTO\Response $response) {
        $validation = null;

        $NRORG  = $this->environment->getNrOrgTrab();

        try {
            $validation = $this->service->validateParamfilipadr($NRORG);

            $response->addDataSet(new DataSet("validation", $validation));
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    /**
     * Procedimento que realiza a requisição de validação
     * da PK de Unidade.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function buildValidationForUnidadePK(DTO\Request\Filter $request, DTO\Response $response) {
        $validation = null;

        $filters = $request->getFilterCriteria()->getConditions();

        $CDFILIAL = $filters[0]['value'];
        $NRORG    = $this->environment->getNrOrgTrab();

        try {
            $validation = $this->service->validateUnidadePK($CDFILIAL, $NRORG);

            $response->addDataSet(new DataSet("validation", $validation));
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    /**
     * Procedimento que deleta o endereço de uma unidade
     * de acordo com os filtros informados.
     *
     * @param Request $request
     * @param Response $response
     */
    public function deleteEndefili(DTO\Request $request, DTO\Response $response) {
        $this->switchDataSource('endefili');

        parent::delete($request, $response);

        $this->switchDataSource('default');
    }

    /**
     * Procedimento que deleta uma Unidade.
     *
     * NOT IMPLEMENTED YET.
     *
     * @param Request $request
     * @param Response $response
     */
    // public function deleteFilial(DTO\Request $request, DTO\Response $response) {
    //     $response->setError(new Error("FILIAL DELETION NOT IMPLEMENTED YET", 500));
    // }

    /**
     * Procedimento que recebe um filtro e realiza a
     * requisição de dados do endereço da unidade de
     * acordo com os filtros informados.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function findEndefili(DTO\Request\Filter $request, DTO\Response $response) {
        $this->switchDataSource('endefili');

        parent::find($request, $response);

        $return = array();
        foreach($response->getDataSets()[0]->getRows() as $row) {
            $row['CDSENHEMAILF'] = ($row['CDSENHEMAILF'] == null)? null : $this->cripto->unCripto($row['CDSENHEMAILF'], 2);
            $return[] = $row;
        }

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet($this->dataSourceNames['endefili'], $return));

        $this->switchDataSource('default');
    }

    /**
     * Procedimento que recebe um filtro e realiza a
     * requisição de dados da unidade de acordo com os
     * filtros informados.
     *
     * Envia no objeto response um DataSet com o
     * resultado ou uma mensagem de erro conforme
     * necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function findFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->switchDataSource('filial');

        parent::find($request, $response);

        $this->switchDataSource('default');
    }

    /**
     * Procedimento que recebe um request e realiza a
     * inserção ou alteração dos dados do endereço da
     * unidade.
     *
     * Envia no objeto response um DataSet com uma
     * mensagem de erro caso necessário.
     *
     * @param Filter $request
     * @param Response $response
     */
    public function saveEndefili(DTO\Request $request, DTO\Response $response) {
        try{
    
            foreach($request->getDataSet()->getRows() as $row) {
                $row = $this->service->validateEndefiliRow($row);
            }
    
            $this->saveGeneric($request, $response, 'endefili');
    
            $this->switchDataSource('default');
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
        
    }

    /**
     * Procedimento que recebe um request e realiza a
     * inserção ou alteração dos dados da unidade.
     *
     * Envia no objeto response um DataSet com uma
     * mensagem de erro caso necessário.
     *
     * Efetua rollback no caso de algum erro.
     *
     * ATENÇÃO! Esse método pressupõe que apenas uma row
     * será recebida! WARNING! AVVERTIMENCO! ADVERTENCIA!
     *
     * @param Filter $request
     * @param Response $response
     */
    public function saveFilial(DTO\Request $request, DTO\Response $response) {
        $this->switchDataSource('filial');
        foreach($request->getDataSet()->getRows() as $row) {
            $row = $this->service->validateUnidadeRow($row);
            $row['NRORG'] = $this->environment->getNrOrgTrab();
        }

        $this->entityManager->beginTransaction();
        try {
            $this->saveGeneric($request, $response, 'filial');

            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new']) {
                    $row = $this->service->validateUnidadeNewRow($row);

                    $this->saveGeneric($request, $response, 'almoxarifado');

                    $this->saveGeneric($request, $response, 'localestoque');

                    $this->saveGeneric($request, $response, 'paramfilial');

                    $NMTABS = array(
                        'ALMOXARIFADO', 'LOCALESTOQUE', 'PARAMFILIAL',
                        'PARAMPEDIDO', 'PARAMPLANEJA', 'PARCMPSUP', 'PAREXPEDICAO'
                    );

                    foreach ($NMTABS as $NMTABPARAMFILI) {
                        $row['NMTABPARAMFILI'] = $NMTABPARAMFILI;

                        $this->saveGeneric($request, $response, 'filipadr');
                    }
                    
                    $this->saveEndefili($request, $response);
                }
            }

            $this->entityManager->commit();
        } catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
            $this->entityManager->rollback();
        }

        $this->switchDataSource('default');
    }
    
    private function saveGeneric ($request, $response, $dataSourceName){
        $this->entityManager->beginTransaction();
        try {
            $this->switchDataSource($dataSourceName);
            parent::save($request, $response);
            $this->entityManager->commit();
        } catch (\Exception $e){
            $this->entityManager->rollback();
            
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
        
    }

    /**
     * Procedimento que troca o DataSource ativo do
     * Controller para o equivalente ao label recebido.
     *
     * @param string label
     */
    private function switchDataSource($label) {
        $this->dataSourceName = $this->dataSourceNames[$label];
    }
}