/**
  * @desc - transforma uma string em objeto javascript
  *         Função semelhante ao JSON.parse.
  * @param string objectString - string de um objeto javascript
  * @return object
  */
function parseJson(objectString) {
    return eval('(' + objectString + ')');
}

/**
  * @desc - realiza a leitura de um arquivo
  * @param string filePath - caminho do arquivo desejado
  * @return string
  */
function readFile(filePath) {
    var fileLoaded = new Packages.java.io.File(filePath);
    var fileRead = new Packages.java.nio.file.Files.lines(fileLoaded.toPath()).collect(Packages.java.util.stream.Collectors.joining());
    return fileRead;
}

/**
  * @desc - realiza a leitura de um arquivo e transforma o retorno desse arquivo em um objeto javascript.
  * @param string filePath - caminho do arquivo desejado
  * @return object
  */
function readAndParseFile(filePath) {
    var file = readFile(filePath);
    var parse = parseJson(file);
    return parse;
}

/**
  * @desc - retorna um objeto das informações utilizadas para acessar a base do relatório
  * @return object
  */
function getConnection() {
    var pathFile = getBasePath() + "/config/connection.json";
    var connection = readAndParseFile(pathFile);
    return connection;
}

/**
  * @desc - retorna o caminho do relatório que está sendo executado
  * @return string
  */
function getBasePath() {
    var text = reportContext.getReportRunnable().getReportName();
    text = text.replace('\\', '/');
    text = (text.indexOf('file:') === -1) ? text : text.substring(text.indexOf('/'), text.length());

    path = text.substring(0, text.lastIndexOf("/"));
    path = path.replace('%20', ' ');

    return path;
}

/**
  * @desc - retornar um parâmetro que está sendo enviado ao relatório.
  * @param string parameterName - nome do parâmetro desejado
  * @return string
  */
function getParameter(parameterName) {
    return reportContext.getParameterValue(parameterName) || '';
}

/**
  * @desc - escreve um arquivo com base na localização do relatório executado
  * @param string text - conteúdo do log que será criado
  */
function writeLog(text) {
    var pathToLog = getBasePath() + '/log.txt';
    writeFile(pathToLog, text);
}

/**
  * @desc - cria ou sobrescreve um arquivo
  * @param string pathFile - caminho completo do arquivo que será criado
  * @param string text     - conteúdo do arquivo que será criado
  */
function writeFile(pathFile, text) {
    var fw = new Packages.java.io.FileWriter(pathFile);
    var bw = new Packages.java.io.BufferedWriter(fw);
    bw.write(text);

    if (bw != null)
        bw.close();

    if (fw != null)
        fw.close();
}

/**
  * @desc - retorna o driver url para acesso à base de dados
  * @return string
  */
function getUrl()  {
    var connection = getConnection();
    return connection.url;
}

/**
  * @desc - retorna o usuário de acesso à base de dados
  * @return string
  */
function getUserName() {
    var connection = getConnection();
    var userName = connection.userName;

    if (connection.useVpd) {
        var org = getParameter('NRORG') || getParameter('P_NRORG');
        userName = 'USR_ORG_' + org;
    }

    return userName;
}

/**
  * @desc - retorna o senha de acesso à base de dados
  * @return string
  */
function getPassword() {
    var connection = getConnection();
    return connection.password;
}

/**
  * @desc - retorna o driver utilizado para o  acesso à base de dados
  * @return string
  */
function getDriverClass() {
    var connection = getConnection();
    return connection.driverClass;
}