function Est32500_ReSuprimentoDeEstoque_Controller(templateManager, ZhLayoutPreferences, ZhFilterPreferences, ScreenService, MessageService, maskEngine, ZhUtil, General, EstUtil) {

	this.onOpenFilter = function(widget) {
		const rowsToSet = {
			defaultFilter: widget.dataSourceFilter,
			REPORT_NAME: "EST325000",
			CDFILIAL: getLocalVar('CDFILIAL'),
			NMFILIAL: getLocalVar('NMFILIAL'),
			DTPOSICAO: getLocalVar('DTPROCESSA'),
			DTSEGU: 0,
			CONSIDERA: 'AMBOS'
		};

		ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
	};

	this.setFilterParams = function(widget, preference = null) {
		const filterRow = preference.FILTER_ROW;

		let filter = [
			{
				"name": "CDFILIAL",
				"operator": "=",
				"value": filterRow.CDFILIAL
            },
			{
				"name": "DTINICIAL",
				"operator": "=",
				"value": filterRow.DTPOSICAO
            },
			{
				"name": "DTFINAL",
				"operator": "=",
				"value": moment(filterRow.DTPOSICAO, "DD/MM/YYYY").add(Number(filterRow.DTSEGU), 'days').format("DD/MM/YYYY")
            },
			{
				"name": "CDPRODINI",
				"operator": "=",
				"value": filterRow.CDPRODINI
            },
			{
				"name": "CDPRODFIN",
				"operator": "=",
				"value": filterRow.CDPRODFIN.padEnd(20, 'Z')
            },
			{
				"name": "DTPOSICAO",
				"operator": "=",
				"value": filterRow.DTPOSICAO
            },
            {
            	"name": "CONSIDERA",
            	"operator": "=",
            	"value": filterRow.CONSIDERA
            }
        ];

		widget.dataSourceFilter = filter;
		widget.reload();
	};

	this.checkProductRange = function(field) {
		var currentRow = field.widget.currentRow;

		if (currentRow.CDPRODINI && currentRow.CDPRODFIN) {
			var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

			if (!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {

				MessageService.showMessageByCode("PRODUTO_INICIAL_MAIOR_FINAL").then(success => {
					currentRow[field.outData[field.descriptionField]] = null;
					currentRow[field.outData[field.valueField]] = null;
					currentRow.CDPRODFIN = null;
					field.widget.getField('NMPRODFIN').clearValue();
					field.widget.getField('NMPRODFIN').reload();
				}).catch(error => {
					console.log(error);
				});
			}
		}
	};

	this.reportProcess = function(args, afterOpenReport = false) {
		const { owner: widget } = args;

		if (!afterOpenReport) {
			const metadata = args.e.metadata;
			const CDFILIAL = EstUtil.getFieldValue('CDFILIAL', widget.name, true);
			const NMFILIAL = EstUtil.getFieldValue('NMFILIAL', widget.name, true);
			const CDPRODINI = EstUtil.getFieldValue('CDPRODINI', widget.name, true);
			const CDPRODFIN = EstUtil.getFieldValue('CDPRODFIN', widget.name, true);
			const mask = {
				type: "fix",
				params: {
					mask: getLocalVar('CDPICTPROD')
				}
			};

			metadata.filter = [
				{
					label: "Unidade",
					value: CDFILIAL + ' - ' + NMFILIAL
				},
				{
					label: "Data Inicial",
					value: EstUtil.getFieldValue('DTPOSICAO', widget.name, true)
				},
				{
					label: "Produto Inicial",
					value: Util.applyMask(CDPRODINI, mask, maskEngine, ZhUtil).split('.').toString().replaceAll(',', '.')
				},
				{
					label: "Produto Final",
					value: Util.applyMask(CDPRODFIN, mask, maskEngine, ZhUtil).split('.').toString().replaceAll(',', '.')
				}
			];
			ZhLayoutPreferences.applyOriginalLayout(widget);
		}
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est32500_ReSuprimentoDeEstoque_Controller', Est32500_ReSuprimentoDeEstoque_Controller);
});
